#!/bin/sh

VERSION=1.2.2

BOARD=$1
UBOOT=$2
KERNEL=$3
DTB=$4
USERLAND=$5

BOARD_LIST="x1 iotg3 iotg3_m1 iotg3_w2 iotg3l degugw_iotg3_m1"

check_params ()
{
	if echo $BOARD_LIST | grep -w -q $BOARD ; then
		INSTALL_IMAGE="install_disk_sd_$(date +%Y%m%d)_${BOARD}.img"
		if [ $BOARD = "degugw_iotg3_m1" ]; then
			BOARD="iotg3_m1"
		fi
		STD_DTB="armadillo_$(echo $BOARD | sed 's/iotg3/iotg_g3/g').dtb"
	else
		echo "ERROR: $BOARD is not supported"
	fi

	IMAGES="$UBOOT $KERNEL $DTB $USERLAND"
	for FILE in $IMAGES; do
		if [ ! -r $FILE ]; then
			echo "ERROR: $FILE: No such file or directory"
			exit 1
		fi
	done
}

get_md5 ()
{
	md5sum $1 | awk '{ print $1 }'
}

create_install_conf ()
{
	echo "#!/bin/sh
#
# install-image configuration
#

#
# Files
#
BOOTLOADER_IMAGE=/images/$(basename $UBOOT)
BOOTLOADER_IMAGE_MD5=$(get_md5 $UBOOT)

KERNEL_IMAGE=/images/uImage
KERNEL_IMAGE_MD5=$(get_md5 $KERNEL)

DEVICETREE_IMAGE=/images/$(basename $STD_DTB)
DEVICETREE_IMAGE_MD5=$(get_md5 $DTB)

ROOTFS_ARCHIVE=/images/$(basename $USERLAND)
ROOTFS_ARCHIVE_MD5=$(get_md5 $USERLAND)

BOARD_UPDATE_TOOL=/images/board-update

PROGRESS=/images/progress-$BOARD.sh"
}

clean ()
{
	rm -rf $TEMP core/p1/boot.scr
	if mountpoint -q $MNTDIR ; then
		umount $MNTDIR
	fi
	rm -rf $MNTDIR
}

create_bootscr ()
{
	if [ $BOARD != "iotg3l" ]; then
		mkimage -A arm -O linux -T script -C none -a 0 -e 0 -d installbootscr core/p1/boot.scr
	fi
}

init_images ()
{
	TEMP=$(mktemp -d)
	mkdir $TEMP/p1 $TEMP/p2

	# Init p1
	cp core/p1/uImage $TEMP/p1/uImage
	if [ $BOARD != "iotg3l" ]; then
		cp core/p1/boot.scr $TEMP/p1/boot.scr
		cp core/p1/x1-minimal.dtb $TEMP/p1/
	else
		cp core/p1/armadillo_iotg_g3l.dtb $TEMP/p1/
	fi

	# Init p2
	tar xzf core/dist.tar.gz -C $TEMP/p2
	cp core/images/* $TEMP/p2/images/
	create_install_conf > $TEMP/p2/images/install.conf
	cp $UBOOT $TEMP/p2/images/
	cp $USERLAND $TEMP/p2/images/
	cp $KERNEL $TEMP/p2/images/uImage
	cp $DTB $TEMP/p2/images/$STD_DTB
}

calc_disk_size_mb ()
{
	# Partition 1: total size of files + 1MB margin
	SIZE_DISK_P1=$(du -sm $TEMP/p1 | awk '{print $1}')
	SIZE_DISK_P1=$(( SIZE_DISK_P1 + 1 ))

	# Partition 2: total size of files + 50MB margin
	SIZE_DISK_P2=$(du -sm $TEMP/p2 | awk '{print $1}')
	SIZE_DISK_P2=$(( SIZE_DISK_P2 + 50 ))

	SIZE_DISK=$(( SIZE_DISK_P1 + SIZE_DISK_P2 ))
}

create_disk ()
{
	rm -f ${INSTALL_IMAGE}

	# Make empty install image
	dd of=${INSTALL_IMAGE} count=0 seek=1 bs=${SIZE_DISK}M

	# Attach install image to loop device
	DISK_DEV_TMP=$(losetup -P -f --show ${INSTALL_IMAGE})

	# Edit partitions
	sfdisk ${DISK_DEV_TMP} <<EOF
,${SIZE_DISK_P1}M,b
,${SIZE_DISK_P2}M,83
EOF

	# Re-attach install image to loop device for partitions
	losetup -d ${DISK_DEV_TMP}
	DISK_DEV_TMP=$(losetup -P -f --show ${INSTALL_IMAGE})

	# Format partitions
	mkfs.vfat ${DISK_DEV_TMP}p1
	mkfs.ext3 -F ${DISK_DEV_TMP}p2

	MNTDIR=$(mktemp -d)

	# Copy files to partition 1
	mount ${DISK_DEV_TMP}p1 $MNTDIR
	cp $TEMP/p1/* $MNTDIR
	sync
	umount $MNTDIR

	# Copy files to partition 2
	mount ${DISK_DEV_TMP}p2 $MNTDIR
	cp -r $TEMP/p2/* $MNTDIR
	sync
	umount $MNTDIR

	rmdir $MNTDIR

	# Write u-boot image
	dd if=core/u-boot-x1.bin of=${DISK_DEV_TMP} bs=1k skip=1 seek=1
	sync

	# Detach loop device
	losetup -d ${DISK_DEV_TMP}
}

usage ()
{
	echo "Install Disk Image Build Script v$VERSION

Usage:
  sudo ./build.sh BOARD UBOOT KERNEL DTB USERLAND

  BOARD: $(echo $BOARD_LIST | sed 's/ /\//g')
  UBOOT: u-boot image
  KERNEL: uImage
  DTB: Device Tree Blob image
  USERLAND: Debian userland archive"
}

if [ $(whoami) != "root" ] ;then
	echo "ERROR: Please run script with sudo"
	exit 1
fi

if [ $# -lt 5 ]; then
	usage
	exit 1
fi

trap 'clean ; exit 1' 1 2 3 15

check_params
create_bootscr
init_images
calc_disk_size_mb
create_disk
clean
